#include "Control.h"

char buffer[125];

Control::~Control()
{ 
   if (os!=NULL && os!=&cout)
    delete os;
}


void Control::SetDefaultParameters()
{
  nrTry = 0;
  max_iter = LONG_MAX;
  maxTry = 1;
  timeLimit = DBL_MAX;
 
#ifdef WIN32
      seed = (int) Timer::getCurrentTime();
#else
      seed = time( NULL );    
#endif

  rg->set_seed(seed);
  init = 1;
  decr = 1;
  k = 1000;
  
  test_version = 0;
  
  Neighborhood = 100;
  solver=1;
  print_statistics = false;
}


void Control::Usage()
{
  cerr<<"\nGraph Coloring\n";
  cerr<<"usage: gcp [- see below -] [dat_file] (given are short options)\n";
  cerr<<"-i f\t inputfile (DIMACS format necessary)\n";
  cerr<<"-o f\t outputfile \n";
  cerr<<"-n #\t number of tests\n";
  cerr<<"-t #\t maximum time\n"; 
  cerr<<"-m #\t number of iterations for each trial\n"; 
  cerr<<"-s #\t seed\n"; 
  cerr<<"-k #\t k for initialization\n"; 
  cerr<<"--solver #\t Solver: \n";
  cerr<<"                      \t 0 => Constructive Heuristics\n";
  cerr<<"--neigh                \t 1 one ex\n";
  cerr<<"                      \t 100 without explorer for heuristics\n";
  cerr<<"--test #\t run test version (1) or not (0)\n";
  cerr<<"--init #\t way of initializing a solution: \n";
  cerr<<"                      \t 0 => completely random - initial k flag to be set!!!\n";
  cerr<<"--stat #\t prints instance statistics\n";
  cerr<<"\nOptional arguments must be separated from their option by a \"=\"";
  cerr<<"character.  Therefore, if \"--max\" accepts an optional argument,\n";
  cerr<<"\"--max 10\" is considered as \"--max\" without argument followed by\n";
  cerr<<"\"10\" which is a non-option command line argument.\n\n";
 cerr<<"--morehelp  \t help: prints this information\n";
}



Control::Control(Random *rnd, int argc, char** argv ) {

  rg = rnd;
  SetDefaultParameters();
  ComParser(argc,argv);
 cerr<<"nrTry "<<nrTry <<endl;
 cerr<<"max_iter "<<max_iter<<endl;
 cerr<<"maxTry "<<maxTry<<endl;
 cerr<<"timeLimit "<<timeLimit<<endl;
 cerr<<"seed "<<seed<<endl;
	
 cerr<<"init "<<init<<endl;
 cerr<<"k "<<k<<endl;
	
 cerr<<"test_version "<<test_version<<endl;
 cerr<<"Neighborhood "<<Neighborhood<<endl;
 cerr<<"solver "<<solver<<endl;
 cerr<<"print_statistics "<<print_statistics <<endl;
}



char * Control::readline (istream &infile) {
  size_t length;
  size_t i;

  //char *result = f
  //cout << "entra/n";
  if (infile.getline(buffer,250))
    { 
      length = strlen (buffer);
      //if (length > 0 && buffer[length-1] != 0) 
      //(*os_log)<<"Buffer size in getline() isn't large enough\n"<<buffer;
      //cout << buffer << endl;
      for (i=0; i<length; ++i) 
	if (buffer[i]!=' ' || buffer[i]!='\t') return buffer;
    }  
  else
    return 0;
  return readline(infile);
}


char * Control::match (char *source, char *prefix) {

  size_t prefixLength = strlen (prefix);

  if (strncmp(source, prefix, prefixLength) != 0) return 0;
  source += prefixLength;
  while ((*source)==' ' || (*source)=='\t' ) source++;
  if (*source == ':' || *source == '=') source++;
  while ((*source)==' ' || (*source)=='\t' ) source++;
  
  return source;
}



void
Control::resetTime() {
	timer.resetTime();
}

double
Control::getTime() {
	return timer.elapsedTime( );
}



void
Control::beginTry() {
	rg->set_seed( seed++ );
	++nrTry;
	(*os) << "begin try " << nrTry << endl;
	resetTime();
	bestCost = MAX_FVALUE;
	best_k = INT_MAX;
	partial_Iter = 0;
}

void
Control::endTry() {
  double time = getTime();
  (*os) << "exit iter " << partial_Iter;
  (*os) << " time ";
  os->flags( ios::fixed );
  (*os) << ( time < 0 ? 0.0 : time ) << endl;
  (*os) << "end try " << nrTry << endl;
  //cout<<bestCost<<"\t"<<seed-1<<endl;
  cout<<best_k<<endl;
  cout<<time<<endl;
}

void
Control::setCurrentCost( fvalue cost, Color cl, unsigned long iter ) {
  if( timeLeft() ) {
     if (cl < best_k)
       {
	 bestCost = cost;
	 best_k=cl;
	 double time = getTime();
	 (*os) << "best " << (int) bestCost;
	 (*os) << " col " << best_k << " time ";
	 os->flags( ios::fixed );
	(*os) << ( time < 0 ? 0.0 : time );
	(*os) << " iter " << (int) iter + partial_Iter;
	(*os) << " par_iter " << (int) iter << endl;
       }
     else if (cl == best_k)
       {
	 if( cost < bestCost ) {
	   bestCost = cost;
	   double time = getTime();
	   (*os) << "best " << (int) bestCost;
	   (*os) << " col " << best_k << " time ";
	   os->flags( ios::fixed );
	   (*os) << ( time < 0 ? 0.0 : time );
	   (*os) << " iter " << (int) iter + partial_Iter;
	  (*os) << " par_iter " << (int) iter << endl;
	 }
       }
  }
}

