#include "Runners.h"

#include <algorithm>
#include <cmath>
#include <iostream>
using namespace std;


//------------------------------------------------------------
//--------------------COLORING RUNNERS------------------------
//------------------------------------------------------------
Runner::Runner(SolutionManager* s,
                               Random *rnd, Control *ctrl, Graph* in = NULL)
{
    p_sm = s;
    p_in = in;
    rg = rnd;
    p_ctrl = ctrl;
    number_of_iterations = 0;

}

Runner::Runner(SolutionManager* s,  NeighborhoodExplorer* e,
                               Random *rnd, Control *ctrl, Graph* in = NULL)
{
    p_sm = s;
    p_ne = e;
    p_in = in;
    rg = rnd;
    p_ctrl = ctrl;
    number_of_iterations = 0;

}

void Runner::InitializeRun()
{
    idle_iterations = 0;
    number_of_iterations = 0;
}

void Runner::initialization_procedure(Coloring &ass)
{
    switch (p_ctrl->init)
    {
    case 0:
        p_sm->randomAssignment(ass,p_ctrl->k);
        break;
    case 1:
        p_sm->constructiveGreedy(ass);
        break;
    default:
        cerr<<"Errors in initialization\n";
        exit(1);
    }
}



void Runner::Run_procedure(Coloring& initial_ass)
{
    switch (p_ctrl->test_version)
    {
    case 0:
        Run(initial_ass);
        break;
    case 1:
        TestRun(initial_ass);
        break;
    default:
        fprintf(stderr,"something wrong in LS!!\n");
        exit(1);
    }
}



void Runner::decreasing_procedure(Coloring &ass)
{
    switch (p_ctrl->decr)
    {
    case 1:
        p_sm->decrease_colors(ass);
        break;
    default:
        cerr<<"something in decreasing_procedure wrong!!\n";
        exit(1);
    }
}

void Runner::TestRun(Coloring &initial_ass)
{
    InitializeRun();
    Coloring current_state(initial_ass);
    Move mv;

    // cout<<current_state;

    while ((current_state.fitness != 0) && !(MaxIterationExpired()) &&
            (idle_iterations < max_idle_iteration) && p_ctrl->timeLeft())
    {
        p_ne->BestMove(current_state,mv);

        if (mv.delta < 0)
        {
            Coloring test_state(current_state);

            if (!(test_state == current_state))
            {
                cerr<<"before"<<endl;
                exit(1);
            }

            cout<<mv<<endl;
            p_ne->MakeMove(current_state,mv);
            //cout << current_state<<endl;
            p_ne->MakeMove(test_state,mv);
            //test_state.col[mv.seq[0]] = mv.seq[1];

            //	  current_state.fitness += mv.delta;

            p_sm->updateColoringData(test_state);
            //cout << test_state<<endl;
            if (!(test_state == current_state))
            {
                exit(EXIT_FAILURE);
            }

            //cout << current_state;

            if (current_state.fitness != 0)
                current_state.feasible = false;
            else
                current_state.feasible = true;

            if (current_state.fitness < initial_ass.fitness)
            {
                initial_ass = current_state;
                p_ctrl->setCurrentCost(current_state.fitness,current_state.CurrColors(),number_of_iterations);
                idle_iterations = 0;
            }

        }
        else
            break;
        idle_iterations++;
        UpdateIterationCounter();
    }
    TerminateRun();
}


void Runner::Run(Coloring &initial_ass)
{
    InitializeRun();
    Coloring current_state(initial_ass);
    Move mv;

    while ((current_state.fitness != 0) && !MaxIterationExpired() &&
            (idle_iterations < max_idle_iteration) && p_ctrl->timeLeft())
    {

        //p_ne->NextMove(current_state,mv);
        p_ne->BestMove(current_state,mv);
        if (mv.delta < 0)
        {
            p_ne->MakeMove(current_state,mv);
            if (current_state.fitness != 0)
                current_state.feasible = false;
            else
            {
                current_state.feasible = true;
            }
            if (current_state.fitness < initial_ass.fitness)
            {
                initial_ass = current_state;

                p_ctrl->setCurrentCost(current_state.fitness,current_state.CurrColors(),number_of_iterations);
                idle_iterations = 0;
            }
        }
        else
            break;
	idle_iterations++;
        UpdateIterationCounter();
    }

    TerminateRun();
}








