#ifndef RUNNERS_H
#define RUNNERS_H


#include <stdio.h>
#include <stdlib.h>
#include <cfloat>
#include <vector>
#include <list>
#include "Input.h"
#include "Control.h"
#include "Random.h"
#include "Solution.h"
#include "SolutionManager.h"
#include "Explorer.h"



class Runner 
{
public:
  Runner(SolutionManager* s, NeighborhoodExplorer* e,
		 Random *rnd, Control *ctrl, Graph* in);

  Runner(SolutionManager* s,
		 Random *rnd, Control *ctrl, Graph* in);
		 
  virtual ~Runner() {};

  void SetCurrentState(const Coloring& s);

  Coloring GetCurrentState();
  fvalue CurrentStateCost();
  Coloring GetBestState();
  fvalue BestStateCost();

  void ComputeCost();

  unsigned long NumberOfIterations() const {return number_of_iterations;};
  void SetMaxIteration(unsigned long max) {max_iteration = max;}
  void SetMaxIdleIteration(unsigned long max) {max_idle_iteration = max;}

  void initialization_procedure(Coloring&);
  void decreasing_procedure(Coloring &ass);
  
  void Run_procedure(Coloring&);

  virtual void Run(Coloring &initial_ass);
  virtual void TestRun(Coloring &initial_ass);

  void setExplorer(NeighborhoodExplorer* e) { p_ne = e;}
  void update(Coloring &a) { p_sm->updateColoringData(a);}
  SolutionManager *getSolutionManager() {return p_sm;};
  NeighborhoodExplorer *p_ne;

protected:
  void InitializeRun() ;
  void TerminateRun() {};
  void ComputeMoveCost();
  void UpdateIterationCounter() {number_of_iterations++;} 
  bool MaxIterationExpired() {return number_of_iterations>max_iteration;}
  
  // input
  Graph* p_in; 
  // helpers
  SolutionManager* p_sm;  

  // state data
  Random *rg;
  Control *p_ctrl;

  unsigned long idle_iterations;
  unsigned long number_of_iterations;

  unsigned long max_iteration; 
  unsigned long max_idle_iteration;
};
 

#endif


