#include <algorithm>
#include <cmath>

#include <iostream>
#include <fstream>
#include "SolutionManager.h"

#define CHARBITS  8

SolutionManager::SolutionManager(Random *rnd, Graph* pin = NULL, Control *ctrl = NULL)
{
  p_in = pin;
  rg = rnd;
  p_ctrl=ctrl;
}

 
fvalue SolutionManager::computeFitness(Coloring &ass)
  {
    ass.fitness = 0;
    for (Vertex v = 0; v < p_in->Vertices(); v++)
    {
      for (Vertex w = v+1; w < p_in->Vertices(); w++)
	{
	  if (p_in->Edge(v,w) && (ass.col[v] == ass.col[w])) 
	    {
	      ass.fitness++;
	    }
	}
    }
    return ass.fitness;
  }



 
void SolutionManager::updateColoringData(Coloring &ass) const 
{
  const Graph &g = *p_in; // g is an alias for the input Graph
  
  ass.fitness = 0;

 for (Vertex i=0; i<g.Vertices(); i++)
    {
      ass.conflicts[i]=0;
 
    }

  for (Vertex i=0; i<g.Vertices(); i++)
  {
    for (Vertex j=i; j<g.Vertices(); j++)
	{
	   if (g.Edge(i,j) && (ass.col[i] == ass.col[j])) 
	    {
	      ass.fitness++;
	      ass.conflicts[i]++;
	      ass.conflicts[j]++;
	    }
	} 
    }
  if (ass.fitness != 0)
    ass.feasible = false;
  else
    ass.feasible = true;
}



void SolutionManager::randomAssignment(Coloring& ass, Color num_col)
{
    Color rnd;
    ass.SetColors(num_col);
    for (Vertex v = 0; v < p_in->Vertices(); v++)
    {
	rnd = (unsigned int) rg->ran0N(num_col-1);
	ass.col[v]=rnd;
    }
    updateColoringData(ass);
}



void SolutionManager::constructiveGreedy(Coloring &ass)
{
  Color num_col = 1;
  Vertex node;
  VertexSet::iterator pt;

  array = rg->generate_array(p_in->Vertices());

  unsigned char **Bitmap;
  unsigned int x = (p_in->Vertices()/CHARBITS+1);
  Bitmap = new unsigned char *[x];
  for (Vertex i = 0; i < x; i++)
    {
      Bitmap[i] = new unsigned char[p_in->Vertices()];
      for (Vertex j = 0; j < (unsigned) (p_in->Vertices()); j++)
	{
	  ass.col[i] = -1;
	  Bitmap[i][j] = '\0';
	}
    }

  
  for(Vertex i = 0; i < p_in->Vertices(); i++) 
    {
      node = array[i];

      Color col = 0;
      while (col<num_col && (Bitmap[(col)/CHARBITS][(node)] & (1<<((col)%CHARBITS))))
	col++;
      
      if (col>=num_col)
	num_col++;

      ass.col[node] = col;
      
      for (pt=p_in->adjacency_set[node].begin();
	   pt!=p_in->adjacency_set[node].end(); pt++)
	{
	  Bitmap[(col)/CHARBITS][(*pt)] |= (1 << ((col) % CHARBITS));
	}
    }
  
  delete [] array;        
  for (Vertex i = 0; i < (p_in->Vertices()/CHARBITS+1); i++)
    delete [] Bitmap[i];
  delete [] Bitmap; 
  ass.SetColors(num_col);
  updateColoringData(ass);
}





void SolutionManager::updateDeltaDataOneEx(Coloring &ass, Vertex var, 
					   Color old_col, Color new_col)
{
 
}




void SolutionManager::decrease_colors(Coloring &ass)
{
  Color removed_col = (Color) rg->ran0N(ass.CurrColors()-1);

  for (Vertex i=0; i < p_in->Vertices(); i++)
    {
      if (ass.col[i]>removed_col)
	{
	  ass.col[i]=ass.col[i]-1;
	}
      else if (ass.col[i]==removed_col)
	{
	  ass.col[i]=rg->ran0N(ass.CurrColors()-2);
	}
    }
  ass.SetColors(ass.CurrColors()-1);
  updateColoringData(ass);
}
