#ifndef SOLVER_H
#define SOLVER_H
#include <vector>
#include "Input.h"
#include "Control.h"
#include "Random.h"
#include "Solution.h"
#include "SolutionManager.h"
#include "Explorer.h"
#include "Runners.h"


  /** The Simple Local Search solver handles a simple local search algorithm
      encapsulated in a runner.
  */

class Solver
  {
  public:
    Solver(Control *c):p_ctrl(c) {}
    virtual ~Solver() {};
    virtual void addRunner(Runner *r) {};
    virtual void Solve(Coloring &current) {}; 
  protected:
    
    Control* p_ctrl;    
 
  };


class ConstructionHeuristics:public Solver
{
  public:
  ConstructionHeuristics(Control* c):Solver(c) {};
  ConstructionHeuristics(Control* c,
		Runner* r):Solver(c), p_runner(r) {};
  virtual ~ConstructionHeuristics() {};
  void Solve(Coloring &current);
 protected:
  Runner* p_runner; /**< A pointer to the managed runner. */
};




class Metaheuristic_Descend:public Solver
{
  public:
  Metaheuristic_Descend(Control* c):Solver(c) {};
  Metaheuristic_Descend(Control* c,
			Runner* r):Solver(c), p_runner(r) {};
  virtual ~Metaheuristic_Descend() {};
  void SetRunner(Runner *r) {p_runner = r;};
  void Solve(Coloring &current);
   protected:
  unsigned long NumberOfIterations() const {return total_iterations;};
  Runner* p_runner; /**< A pointer to the managed runner. */
  bool MaxIterationExpired() {return total_iterations>p_ctrl->max_iter;}
  unsigned long total_iterations; /**< The overall number of iterations
				     performed by all the managed 
			     runners. */
};


#endif
