#include "Timer.h"

#ifdef WIN32
// Windows Version
#include <sys/timeb.h>

double Timer::getCurrentTime() 
{
  _timeb sysTime;
  _ftime(&sysTime);
  return (double) sysTime.time + (double) sysTime.millitm * 1.0E-3;
}

void Timer::setAlarm(int seconds, void (*signalHandler)(int))
{
}

#else

// Linux Version
#include <sys/time.h>
#include <sys/resource.h>
#include <csignal>
#include <unistd.h>

// get the VIRTUAL time
double Timer::getCurrentTime() 
{
  struct rusage res;
  getrusage( RUSAGE_SELF, &res );
  return (double) res.ru_utime.tv_sec +
    (double) res.ru_stime.tv_sec +
    (double) res.ru_utime.tv_usec * 1.0E-6 +
    (double) res.ru_stime.tv_usec * 1.0E-6;
}



void Timer::setAlarm(int seconds, void (*signalHandler)(int))
{
  if (seconds > 0) 
  {
    signal(SIGALRM, signalHandler);
    resetTime();
  }
  alarm(seconds);
}


#endif


Timer::Timer()
{
  resetTime();
}

void Timer::resetTime()
{
  startTime = getCurrentTime();
}

double Timer::elapsedTime() const
{
  return getCurrentTime() - startTime;
}


