#include <cmath>
#include <cfloat>
#include <string>
#include<fstream>

//#include <unistd.h>
#include <csignal>

#include <sys/time.h>
#include <sys/resource.h>


#include "Timer.h"
#include "Random.h"
#include "Utilities.h"
#include "Input.h"
#include "Solution.h"
#include "SolutionManager.h"
#include "Explorer.h"
#include "Runners.h"
#include "Solver.h"

using namespace std;


void got_timeout(int s) {
 cerr<<"Interrupted. Time Limit exceeded!\n";
 exit(EXIT_FAILURE);  
}
 

long int *array;

int main( int argc, char** argv ) {

    //Create a control object from the command line arguments.
    Random random(0);
    Control control(&random, argc, argv );

    control.resetTime();

    Graph graph;
    graph.Load(control.getInputStream(),control.print_statistics);
    control.getOutputStream()<<"Read instance. Time: "<<control.getTime()<<endl;
    
    while (control.triesLeft())
    {
        SolutionManager *coloringManager = new SolutionManager(&random, &graph, &control);
        //NeighborhoodExplorer *neighExp = new NeighborhoodExplorer(coloringManager, &random, &graph);
	OneExExplorer *neighExp = new OneExExplorer(coloringManager, &random, &graph);
        //Runner *coloringRunner = new Runner(coloringManager,&random,&control,&graph); 
        Runner *coloringRunner = new Runner(coloringManager,neighExp,&random,&control,&graph); 
        Solver *coloringSolver = NULL;// = new Solver(&control, coloringRunner);
        
	switch (control.solver)
        {
	case 1:
	  coloringSolver = new ConstructionHeuristics(&control, coloringRunner);
	  break;
	case 2:
	  coloringRunner->SetMaxIdleIteration(LONG_MAX);//100 * graph.Vertices() * coloring.k);
	  coloringRunner->SetMaxIteration(control.max_iter);//LONG_MAX);
     	  coloringSolver = new Metaheuristic_Descend(&control, coloringRunner);
	  break;
        default:
	  cerr<<"Wrong solver setting\n";
	  exit(EXIT_FAILURE);
        }


        //------------------------------------------------------------
        control.beginTry();
        Coloring *coloring = new Coloring(&random, &graph);
        coloringSolver->Solve(*coloring);
        control.endTry();
        //------------------------------------------------------------

	if (FALSE)
	  {
	    control.getOutputSolutionStream()<<(*coloring);//ooo<<original_coloring;
	  }

	delete coloring;
        delete coloringSolver;
        delete coloringRunner;
        delete coloringManager;

    }
    return EXIT_SUCCESS;
}

